@@ -48,10 +48,12 @@ def _get_configured_scraper(scraper_type, options, metadata):
                         strict_validation=options.strict,
                         fastmode=options.fastmode)
 
+
 def _is_old_scrape(f):
     argspec = inspect.getargspec(f)
     return 'chamber' in argspec.args
 
+
 def _run_scraper(scraper_type, options, metadata):
     """
         scraper_type: bills, legislators, committees, votes
@@ -66,7 +68,6 @@ def _run_scraper(scraper_type, options, metadata):
             "end_time": dt.datetime.utcnow()
         }]
 
-
     runs = []
 
     # Removed from the inner loop due to non-bicameral scrapers
@@ -78,7 +79,7 @@ def _run_scraper(scraper_type, options, metadata):
     if scraper_type in ('bills', 'votes', 'events'):
         times = options.sessions
         for time in times:
-            scraper.validate_session(time)
+            scraper.validate_session(time, scraper.latest_only)
     elif scraper_type in ('committees', 'legislators'):
         times = options.terms
         for time in times:
@@ -133,6 +134,7 @@ def _do_imports(abbrev, args):
     dist_filename = os.path.join(settings.BILLY_MANUAL_DATA_DIR, 'districts',
                                  '%s.csv' % abbrev)
     if os.path.exists(dist_filename):
+        db.districts.remove({'abbr': abbrev})
         dist_csv = unicodecsv.DictReader(open(dist_filename))
         for dist in dist_csv:
             dist['_id'] = '%(abbr)s-%(chamber)s-%(name)s' % dist
@@ -251,11 +253,11 @@ def main(old_scrape_compat=False):
         # configure oyster
         if settings.ENABLE_OYSTER:
             from oyster.conf import settings as oyster_settings
-            oyster_settings.DOCUMENT_CLASSES[args.module + ':billtext'] = module.document_class
+            oyster_settings.DOCUMENT_CLASSES[args.module + ':billtext'] = \
+                    module.document_class
 
         # make output dir
         args.output_dir = os.path.join(settings.BILLY_DATA_DIR, abbrev)
-        _clear_scraped_data(args.output_dir)
 
         # if terms aren't set, use latest
         if not args.terms:
@@ -306,6 +308,8 @@ def main(old_scrape_compat=False):
         scrape_data = {}
 
         if 'scrape' in args.actions:
+            _clear_scraped_data(args.output_dir)
+
             # validate then write metadata
             if hasattr(module, 'session_list'):
                 session_list = module.session_list()
@@ -343,13 +347,14 @@ def main(old_scrape_compat=False):
 
             # scraper order matters
             order = ('legislators', 'committees', 'votes', 'bills', 'events')
+            _traceback = None
             try:
                 for stype in order:
                     if stype in args.types:
                         run_record += _run_scraper(stype, args, metadata)
             except Exception as e:
                 _traceback = _, _, exc_traceback = sys.exc_info()
-                run_record += [{"exception": e, "type": stype }]
+                run_record += [{"exception": e, "type": stype}]
                 lex = e
 
             exec_end = dt.datetime.utcnow()
