@@ -1,31 +1,30 @@
 import os
 import json
 import random
-import subprocess
 
 from billy import db
 from billy.commands import BaseCommand
 from billy.utils import metadata
 
-from billy.conf import settings, base_arg_parser
 from billy.commands.dump import APIValidator, api_url
 
 import scrapelib
 import validictory
 
+
 class ValidateApi(BaseCommand):
     name = 'validateapi'
     help = 'validate data from the API'
 
     def add_args(self):
-        self.add_argument('--sunlight_key', dest='SUNLIGHT_SERVICES_KEY',
-                  help='the Sunlight API key to use')
+        self.add_argument('--apikey', dest='API_KEY',
+                  help='the API key to use')
         self.add_argument('--schema_dir', default=None,
                   help='directory to use for API schemas (optional)')
 
     def handle(self, args):
-        for metadata in db.metadata.find():
-            validate_api(metadata['abbreviation'], args.schema_dir)
+        for meta in db.metadata.find():
+            validate_api(meta['abbreviation'], args.schema_dir)
 
 
 def get_json_schema(name, schema_dir):
