@@ -1,18 +1,26 @@
-#!/usr/bin/env python
 import os
-import re
 import glob
 import json
 import logging
+from time import time
 from collections import defaultdict
 
 from billy.conf import settings
 from billy.utils import metadata, term_for_session
+from billy.scrape import JSONDateEncoder
 from billy import db
 from billy.importers.names import get_legislator_id
 from billy.importers.subjects import SubjectCategorizer
 from billy.importers.utils import (insert_with_id, update, prepare_obj,
-                                   next_big_id, oysterize, fix_bill_id)
+                                   next_big_id, oysterize, fix_bill_id,
+                                   get_committee_id)
+
+if hasattr(settings, "ENABLE_GIT") and settings.ENABLE_GIT:
+    from dulwich.repo import Repo
+    from dulwich.objects import Blob
+    from dulwich.objects import Tree
+    from dulwich.objects import Commit, parse_timezone
+
 
 import pymongo
 
@@ -20,25 +28,45 @@ logger = logging.getLogger('billy')
 
 
 def ensure_indexes():
-    # accomodates basic search / unique constraint
+    # TODO: add a _current_term, _current_session index?
+
+    # accomodates basic lookup / unique constraint on state/session/bill_id
     db.bills.ensure_index([('state', pymongo.ASCENDING),
                            ('session', pymongo.ASCENDING),
                            ('chamber', pymongo.ASCENDING),
                            ('bill_id', pymongo.ASCENDING)],
                           unique=True)
-    # used for search in conjunction with ElasticSearch
-    db.bills.ensure_index([('versions.doc_id', pymongo.ASCENDING),
-                           ('created_at', pymongo.DESCENDING)])
-    # TODO: add a _current_term, _current_session index
-    # TODO: re-evaluate if the below indices are needed
+
+    # doc_id is used for search in conjunction with ElasticSearch
+    #  sort field (date) comes first, followed by field that we do an $in on
+    db.bills.ensure_index([('created_at', pymongo.DESCENDING),
+                           ('versions.doc_id', pymongo.ASCENDING)])
+    db.bills.ensure_index([('updated_at', pymongo.DESCENDING),
+                           ('versions.doc_id', pymongo.ASCENDING)])
+    db.bills.ensure_index([('action_dates.last', pymongo.DESCENDING),
+                           ('versions.doc_id', pymongo.ASCENDING)])
+
+    # common search indices
     db.bills.ensure_index([('state', pymongo.ASCENDING),
-                           ('session', pymongo.ASCENDING),
-                           ('chamber', pymongo.ASCENDING),
-                           ('subjects', pymongo.ASCENDING)])
+                           ('subjects', pymongo.ASCENDING),
+                           ('action_dates.last', pymongo.DESCENDING)])
     db.bills.ensure_index([('state', pymongo.ASCENDING),
-                           ('session', pymongo.ASCENDING),
-                           ('chamber', pymongo.ASCENDING),
-                           ('sponsors.leg_id', pymongo.ASCENDING)])
+                           ('sponsors.leg_id', pymongo.ASCENDING),
+                           ('action_dates.last', pymongo.DESCENDING)])
+
+    # generic sort-assist indices on the action_dates
+    db.bills.ensure_index([('state', pymongo.ASCENDING),
+                           ('action_dates.first', pymongo.DESCENDING)])
+    db.bills.ensure_index([('state', pymongo.ASCENDING),
+                           ('action_dates.last', pymongo.DESCENDING)])
+    db.bills.ensure_index([('state', pymongo.ASCENDING),
+                           ('action_dates.passed_upper', pymongo.DESCENDING)])
+    db.bills.ensure_index([('state', pymongo.ASCENDING),
+                           ('action_dates.passed_lower', pymongo.DESCENDING)])
+
+    # votes index
+    db.votes.ensure_index([('_voters', pymongo.ASCENDING),
+                           ('date', pymongo.ASCENDING)])
 
 
 def import_votes(data_dir):
@@ -59,6 +87,119 @@ def import_votes(data_dir):
     logger.info('imported %s vote files' % len(paths))
     return votes
 
+
+git_active_repo = None
+git_active_commit = None
+git_active_tree = None
+git_old_tree = None
+HEAD = None
+
+
+def git_add_bill(data):
+    if not hasattr(settings, "ENABLE_GIT") or not settings.ENABLE_GIT:
+        return
+
+    global git_active_repo
+    global git_active_tree
+    global git_active_commit
+
+    bill = json.dumps(data,
+                      cls=JSONDateEncoder,
+                      sort_keys=True,
+                      indent=4)
+    spam = Blob.from_string(bill)
+    bid = str(data['_id'])
+    git_active_repo.object_store.add_object(spam)
+    git_active_tree[bid] = (0100644, spam.id)
+    git_active_tree.check()
+    print "added %s - %s" % (data['_id'], spam.id)
+
+
+def git_commit(message):
+    if not hasattr(settings, "ENABLE_GIT") or not settings.ENABLE_GIT:
+        return
+
+    print "Commiting import as '%s'" % (message)
+
+    global git_active_repo
+    global git_active_tree
+    global git_old_tree
+    global git_active_commit
+    global HEAD
+    repo = git_active_repo
+
+    if git_old_tree == git_active_tree.id:
+        # We don't wait t commit twice.
+        print "Nothing new here. Bailing out."
+        return
+
+    c = git_active_commit
+    c.tree = git_active_tree.id
+    c.parents = [HEAD]
+    repo.object_store.add_object(git_active_tree)
+    c.author = c.committer = "Billy <openstates@sunlightfoundation.com>"
+    c.commit_time = c.author_time = int(time())
+    tz = parse_timezone("-0400")[0]
+    c.commit_timezone = c.author_timezone = tz
+    c.encoding = "UTF-8"
+    c.message = message
+    repo.object_store.add_object(c)
+    repo.refs['refs/heads/master'] = c.id
+
+
+def git_repo_init(gitdir):
+    os.mkdir(gitdir)
+    repo = Repo.init_bare(gitdir)
+    blob = Blob.from_string("""Why, Hello there!
+
+This is your friendly Legislation tracker, Billy here.
+
+This is a git repo full of everything I write to the DB. This isn't super
+useful unless you're debugging production issues.
+
+Fondly,
+   Bill, your local Billy instance.""")
+    tree = Tree()
+    tree.add("README", 0100644, blob.id)
+    commit = Commit()
+    commit.tree = tree.id
+    author = "Billy <openstates@sunlightfoundation.com>"
+    commit.author = commit.committer = author
+    commit.commit_time = commit.author_time = int(time())
+    tz = parse_timezone('-0400')[0]
+    commit.commit_timezone = commit.author_timezone = tz
+    commit.encoding = "UTF-8"
+    commit.message = "Initial commit"
+    repo.object_store.add_object(blob)
+    repo.object_store.add_object(tree)
+    repo.object_store.add_object(commit)
+    repo.refs['refs/heads/master'] = commit.id
+
+
+def git_prelod(abbr):
+    if not hasattr(settings, "ENABLE_GIT") or not settings.ENABLE_GIT:
+        return
+
+    global git_active_repo
+    global git_active_commit
+    global git_active_tree
+    global git_old_tree
+    global HEAD
+
+    gitdir = "%s/%s.git" % (settings.GIT_PATH, abbr)
+
+    if not os.path.exists(gitdir):
+        git_repo_init(gitdir)
+
+    git_active_repo = Repo(gitdir)
+    git_active_commit = Commit()
+    HEAD = git_active_repo.head()
+    commit = git_active_repo.commit(HEAD)
+    tree = git_active_repo.tree(commit.tree)
+    git_old_tree = tree.id
+    git_active_tree = tree
+
+
 def oysterize_version(bill, version):
     titles = [bill['title']] + bill.get('alternate_titles', [])
     logger.info('{0} tracked in oyster'.format(version['doc_id']))
@@ -128,10 +269,17 @@ def import_bill(data, votes, categorizer):
 
     # match sponsor leg_ids
     for sponsor in data['sponsors']:
-        id = get_legislator_id(abbr, data['session'], None,
+        # use sponsor's chamber if specified
+        id = get_legislator_id(abbr, data['session'], sponsor.get('chamber'),
                                sponsor['name'])
         sponsor['leg_id'] = id
+        if id is None:
+            cid = get_committee_id(level, abbr, data['chamber'],
+                                   sponsor['name'])
+            if not cid is None:
+                sponsor['committee_id'] = cid
 
+    # process votes
     for vote in data['votes']:
 
         # committee_ids
@@ -150,6 +298,57 @@ def import_bill(data, votes, categorizer):
 
             vote[vtype] = svlist
 
+    # process actions
+    dates = {'first': None, 'last': None, 'passed_upper': None,
+             'passed_lower': None, 'signed': None}
+    for action in data['actions']:
+        adate = action['date']
+
+        def _match_committee(name):
+            return get_committee_id(level, abbr, action['actor'], name)
+
+        def _match_legislator(name):
+            return get_legislator_id(abbr,
+                                     data['session'],
+                                     action['actor'],
+                                     name)
+
+        resolvers = {
+            "committee": _match_committee,
+            "legislator": _match_legislator
+        }
+
+        if "related_entities" in action:
+            for entity in action['related_entities']:
+                try:
+                    resolver = resolvers[entity['type']]
+                except KeyError as e:
+                    # We don't know how to deal.
+                    logger.error("I don't know how to sort a %s" % e)
+                    continue
+
+                id = resolver(entity['name'])
+                entity['id'] = id
+
+        # first & last
+        if not dates['first'] or adate < dates['first']:
+            dates['first'] = adate
+        if not dates['last'] or adate > dates['last']:
+            dates['last'] = adate
+
+        # passed & signed
+        if (not dates['passed_upper'] and action['actor'] == 'upper'
+            and 'bill:passed' in action['type']):
+            dates['passed_upper'] = adate
+        elif (not dates['passed_lower'] and action['actor'] == 'lower'
+            and 'bill:passed' in action['type']):
+            dates['passed_lower'] = adate
+        elif (not dates['signed'] and 'governor:signed' in action['type']):
+            dates['signed'] = adate
+
+    # save action dates to data
+    data['action_dates'] = dates
+
     data['_term'] = term_for_session(abbr, data['session'])
 
     alt_titles = set(data.get('alternate_titles', []))
@@ -173,10 +372,14 @@ def import_bill(data, votes, categorizer):
     data['alternate_titles'] = list(alt_titles)
 
     if not bill:
-        insert_with_id(data)
+        bill_id = insert_with_id(data)
+        git_add_bill(data)
+        denormalize_votes(data, bill_id)
         return "insert"
     else:
+        git_add_bill(bill)
         update(bill, data, db.bills)
+        denormalize_votes(bill, bill['_id'])
         return "update"
 
 
@@ -184,6 +387,8 @@ def import_bills(abbr, data_dir):
     data_dir = os.path.join(data_dir, abbr)
     pattern = os.path.join(data_dir, 'bills', '*.json')
 
+    git_prelod(abbr)
+
     counts = {
         "update": 0,
         "insert": 0,
@@ -216,10 +421,13 @@ def import_bills(abbr, data_dir):
     level = meta['level']
     populate_current_fields(level, abbr)
 
+    git_commit("Import Update")
+
     ensure_indexes()
 
     return counts
 
+
 def populate_current_fields(level, abbr):
     """
     Set/update _current_term and _current_session fields on all bills
@@ -243,6 +451,22 @@ def populate_current_fields(level, abbr):
         db.bills.save(bill, safe=True)
 
 
+def denormalize_votes(bill, bill_id):
+    # remove all existing votes for this bill
+    db.votes.remove({'bill_id': bill_id}, safe=True)
+
+    # add votes
+    for vote in bill.get('votes', []):
+        vote = vote.copy()
+        vote['_id'] = vote['vote_id']
+        vote['bill_id'] = bill_id
+        vote['state'] = bill['state']
+        vote['_voters'] = [l['leg_id'] for l in vote['yes_votes']]
+        vote['_voters'] += [l['leg_id'] for l in vote['no_votes']]
+        vote['_voters'] += [l['leg_id'] for l in vote['other_votes']]
+        db.votes.save(vote, safe=True)
+
+
 class GenericIDMatcher(object):
 
     def __init__(self, abbr):
@@ -297,28 +521,3 @@ class DocumentMatcher(GenericIDMatcher):
     def key_for_item(self, document):
         # URL is good enough as a key
         return (document['url'],)
-
-
-__committee_ids = {}
-
-
-def get_committee_id(level, abbr, chamber, committee):
-    key = (level, abbr, chamber, committee)
-    if key in __committee_ids:
-        return __committee_ids[key]
-
-    spec = {'level': level, level: abbr, 'chamber': chamber,
-            'committee': committee, 'subcommittee': None}
-
-    comms = db.committees.find(spec)
-
-    if comms.count() != 1:
-        spec['committee'] = 'Committee on ' + committee
-        comms = db.committees.find(spec)
-
-    if comms and comms.count() == 1:
-        __committee_ids[key] = comms[0]['_id']
-    else:
-        __committee_ids[key] = None
-
-    return __committee_ids[key]
