@@ -9,13 +9,15 @@ import pymongo.errors
 import name_tools
 
 from billy import db
+from billy.conf import settings
 
-oyster_import_exception = None
-try:
-    from oyster.core import kernel
-except ImportError as e:
-    kernel = None
-    oyster_import_exception = e
+if settings.ENABLE_OYSTER:
+    oyster_import_exception = None
+    try:
+        from oyster.core import kernel
+    except ImportError as e:
+        kernel = None               # noqa
+        oyster_import_exception = e
 
 
 def _get_property_dict(schema):
@@ -70,12 +72,13 @@ def insert_with_id(obj):
     else:
         raise ValueError("unknown _type for object")
 
-    level = obj[obj['level']].upper()
+    # get abbr from level
+    abbr = obj[obj['level']].upper()
 
-    id_reg = re.compile('^%s%s' % (level, id_type))
+    id_reg = re.compile('^%s%s' % (abbr, id_type))
 
     # Find the next available _id and insert
-    id_prefix = '%s%s' % (level, id_type)
+    id_prefix = '%s%s' % (abbr, id_type)
     cursor = collection.find({'_id': id_reg}).sort('_id', -1).limit(1)
 
     try:
@@ -98,6 +101,7 @@ def insert_with_id(obj):
         except pymongo.errors.DuplicateKeyError:
             new_id += 1
 
+
 def _timestamp_to_dt(timestamp):
     tstruct = time.localtime(timestamp)
     dt = datetime.datetime(*tstruct[0:6])
@@ -120,10 +124,20 @@ def compare_committee(ctty1, ctty2):
         for junk in ctty_junk_words:
             obj = re.sub(junk, " ", obj).strip()
         obj = re.sub("\s+", " ", obj)
+        obj = re.sub(r'\s+', ' ', re.sub(r'\W+', ' ', obj)).strip()
         return obj
-    ctty1 = _cleanup(ctty1)
-    ctty2 = _cleanup(ctty2)
-    return ctty1 == ctty2
+    check_both = [
+        ("", ""),
+        ("&", "and")
+    ]
+    for old, new in check_both:
+        c1 = ctty1.replace(old, new)
+        c2 = ctty2.replace(old, new)
+        c1 = _cleanup(c1)
+        c2 = _cleanup(c2)
+        if c1 == c2:
+            return True
+    return False
 
 
 def update(old, new, collection, sneaky_update_filter=None):
@@ -323,20 +337,75 @@ def merge_legislators(leg1, leg2):
         #      old_roles & roles!! There's a potenital for data loss, but it's
         #      not that big of a thing.
         #   -- paultag & jamesturk, 02-02-2012
-        crole = leg1[roles][0]
-        try:
-            leg1[old_roles][crole['term']].append(crole)
-        except KeyError:
+        if len(leg1[roles]) > 0:
+            crole = leg1[roles][0]
             try:
-                leg1[old_roles][crole['term']] = [crole]
+                leg1[old_roles][crole['term']].append(crole)
             except KeyError:
-                # dear holy god this needs to be fixed.
-                leg1[old_roles] = {crole['term']: [crole]}
+                try:
+                    leg1[old_roles][crole['term']] = [crole]
+                except KeyError:
+                    # dear holy god this needs to be fixed.
+                    leg1[old_roles] = {crole['term']: [crole]}
+
+        if len(leg2[roles]) > 0:
+            # OK. We've migrated the newly old roles to the old_roles entry.
+            leg1[roles] = [leg2[roles][0]]
 
-        # OK. We've migrated the newly old roles to the old_roles entry.
-        leg1[roles] = [leg2[roles][0]]
     return (leg1, leg2['_id'])
 
+__committee_ids = {}
+
+
+def get_committee_id(level, abbr, chamber, committee):
+    key = (level, abbr, chamber, committee)
+    if key in __committee_ids:
+        return __committee_ids[key]
+
+    spec = {'level': level, level: abbr, 'chamber': chamber,
+            'committee': committee, 'subcommittee': None}
+
+    comms = db.committees.find(spec)
+
+    if comms.count() != 1:
+        flag = 'Committee on'
+        if flag not in committee:
+            spec['committee'] = 'Committee on ' + committee
+        else:
+            spec['committee'] = committee.replace(flag, "").strip()
+        comms = db.committees.find(spec)
+
+    if comms and comms.count() == 1:
+        __committee_ids[key] = comms[0]['_id']
+    else:
+        # last resort :(
+        comm_id = get_committee_id_alt(level, abbr, committee, chamber)
+        __committee_ids[key] = comm_id
+
+    return __committee_ids[key]
+
+
+def get_committee_id_alt(level, abbr, name, chamber):
+    matched_committee = None
+    spec = {"state": abbr, "chamber": chamber}
+    if chamber is None:
+        del(spec['chamber'])
+    comms = db.committees.find(spec)
+    for committee in comms:
+        c = committee['committee']
+        if committee['subcommittee'] is not None:
+            c += " %s" % (committee['subcommittee'])
+
+        if compare_committee(name, c):
+            if not matched_committee is None:
+                return None  # In the event we match more then one committee.
+            matched_committee = committee['_id']
+
+    if matched_committee is None and not chamber is None:
+        matched_committee = get_committee_id_alt(level, abbr, name, None)
+
+    return matched_committee
+
 
 def oysterize(url, doc_class, id, **kwargs):
     if not kernel:
