@@ -4,7 +4,7 @@ from collections import defaultdict
 
 from billy import db
 from billy.utils import term_for_session
-from billy.reports.utils import update_common
+from billy.reports.utils import update_common, get_quality_exceptions
 
 logger = logging.getLogger('billy')
 
@@ -24,7 +24,7 @@ def _bill_report_dict():
             'actions_per_month': defaultdict(int),
             'sponsorless_count': 0,
             '_sponsor_count': 0,
-            '_sponsors_with_leg_id_count': 0,
+            '_sponsors_with_id_count': 0,
             'sponsors_per_type': defaultdict(int),
             'vote_count': 0,
             '_passed_vote_count': 0,
@@ -36,7 +36,6 @@ def _bill_report_dict():
             '_rollcalls_with_leg_id_count': 0,
             '_subjects_count': 0,
             'bills_per_subject': defaultdict(int),
-            'sourceless_count': 0,
             'versionless_count': 0,
             'version_count': 0,
             'unmatched_leg_ids': set(),
@@ -53,6 +52,9 @@ def scan_bills(abbr):
     uncategorized_subjects = defaultdict(int)
     sessions = defaultdict(_bill_report_dict)
 
+    # load exception data into sets of ids indexed by exception type
+    quality_exceptions = get_quality_exceptions(abbr)
+
     for bill in db.bills.find({'level': level, level: abbr}):
         session_d = sessions[bill['session']]
 
@@ -79,14 +81,19 @@ def scan_bills(abbr):
                 other_actions[action['action']] += 1
             session_d['actions_per_actor'][action['actor']] += 1
             session_d['actions_per_month'][date.strftime('%Y-%m')] += 1
+
+        # handle no_actions bills
         if not bill['actions']:
-            session_d['actionless_count'] += 1
+            if bill['_id'] not in quality_exceptions['bills:no_actions']:
+                session_d['actionless_count'] += 1
+            else:
+                quality_exceptions['bills:no_actions'].remove(bill['_id'])
 
         # sponsors
         for sponsor in bill['sponsors']:
             session_d['_sponsor_count'] += 1
-            if sponsor.get('leg_id'):
-                session_d['_sponsors_with_leg_id_count'] += 1
+            if sponsor.get('leg_id') or sponsor.get('committee_id'):
+                session_d['_sponsors_with_id_count'] += 1
             else:
                 # keep missing leg_ids
                 session_d['unmatched_leg_ids'].add(
@@ -94,8 +101,13 @@ def scan_bills(abbr):
                     sponsor['name'])
                 )
             session_d['sponsors_per_type'][sponsor['type']] += 1
+
+        # handle no sponsors bills
         if not bill['sponsors']:
-            session_d['sponsorless_count'] += 1
+            if bill['_id'] not in quality_exceptions['bills:no_sponsors']:
+                session_d['sponsorless_count'] += 1
+            else:
+                quality_exceptions['bills:no_sponsors'].remove(bill['_id'])
 
         # votes
         for vote in bill['votes']:
@@ -129,11 +141,19 @@ def scan_bills(abbr):
                     )
 
             # check counts if any rollcalls are present
-            if (has_rollcalls and
-                (len(vote['yes_votes']) != vote['yes_count'] or
-                 len(vote['no_votes']) != vote['no_count'] or
-                 len(vote['other_votes']) != vote['other_count'])):
-                session_d['bad_vote_counts'].add(bill['_id'])
+            if has_rollcalls:
+                if (len(vote['yes_votes']) != vote['yes_count'] and
+                    vote['vote_id'] not in
+                    quality_exceptions['votes:bad_yes_count']):
+                    session_d['bad_vote_counts'].add(vote['vote_id'])
+                if (len(vote['no_votes']) != vote['no_count'] and
+                    vote['vote_id'] not in
+                    quality_exceptions['votes:bad_no_count']):
+                    session_d['bad_vote_counts'].add(vote['vote_id'])
+                if (len(vote['other_votes']) != vote['other_count'] and
+                    vote['vote_id'] not in
+                    quality_exceptions['votes:bad_other_count']):
+                    session_d['bad_vote_counts'].add(vote['vote_id'])
 
         # subjects
         for subj in bill.get('scraped_subjects', []):
@@ -150,13 +170,16 @@ def scan_bills(abbr):
         # versions
         if not bill['versions']:
             # total num of bills w/o versions
-            session_d['versionless_count'] += 1
+            if bill['_id'] not in quality_exceptions['bills:no_versions']:
+                session_d['versionless_count'] += 1
+            else:
+                quality_exceptions['bills:no_versions'].remove(bill['_id'])
         else:
             # total num of versions
             session_d['version_count'] += len(bill['versions'])
         for doc in bill['versions']:
             duplicate_versions[doc['url']] += 1
-        # TODO: add a duplicate documents back in?
+        # TODO: add duplicate document detection back in?
 
     dup_version_urls = []
     dup_source_urls = []
@@ -167,6 +190,12 @@ def scan_bills(abbr):
         if n > 1:
             dup_source_urls.append(url)
 
+    # do logging of unnecessary exceptions
+    for qe_type, qes in quality_exceptions.iteritems():
+        if qes:
+            logger.warning('unnecessary {0} exceptions for {1} bills: \n  {2}'
+                           .format(qe_type, len(qes), '\n  '.join(qes)))
+
     return {'duplicate_versions': dup_version_urls,
             'duplicate_sources': dup_source_urls,
             'other_actions': other_actions.items(),
@@ -222,8 +251,8 @@ def calculate_percentages(report):
     # sponsors
     _sponsor_count = float(report.pop('_sponsor_count')) / 100
     if _sponsor_count:
-        report['sponsors_with_leg_id'] = (
-            report.pop('_sponsors_with_leg_id_count') / _sponsor_count)
+        report['sponsors_with_id'] = (
+            report.pop('_sponsors_with_id_count') / _sponsor_count)
         for k in report['sponsors_per_type'].iterkeys():
             report['sponsors_per_type'][k] /= _sponsor_count
 
