@@ -13,7 +13,8 @@ class LegislatorScraper(Scraper):
                                    '../schemas/person.json')
         schema = json.load(open(schema_path))
         terms = [t['name'] for t in self.metadata['terms']]
-        schema['properties']['roles']['items']['properties']['term']['enum'] = terms
+        schema['properties']['roles']['items']['properties']['term']['enum'] =\
+                terms
         return schema
 
     def scrape(self, chamber, term):
@@ -72,6 +73,7 @@ class Person(SourcedObject):
         self['middle_name'] = middle_name
         self['suffixes'] = kwargs.get('suffixes', '')
         self['roles'] = []
+        self['offices'] = []
 
     def add_role(self, role, term, start_date=None, end_date=None,
                  **kwargs):
@@ -88,6 +90,17 @@ class Person(SourcedObject):
                                   start_date=start_date,
                                   end_date=end_date, **kwargs))
 
+    def add_office(self, type, name, address=None, phone=None, fax=None,
+                   email=None):
+        """
+        Allowed office types:
+            capitol
+            district
+        """
+        office_dict = dict(type=type, address=address, name=name, phone=phone,
+                           fax=fax, email=email)
+        self['offices'].append(office_dict)
+
     def get_filename(self):
         role = self['roles'][0]
         filename = "%s_%s.json" % (role['term'], self['full_name'])
