@@ -2,14 +2,16 @@ import copy
 from billy import db
 from billy.importers import bills, names
 
-from nose.tools import with_setup
+from nose.tools import with_setup, assert_equal
 
 
 def setup_func():
     db.metadata.drop()
     db.bills.drop()
     db.legislators.drop()
+    db.document_ids.drop()
     db.vote_ids.drop()
+    db.committees.drop()
     names.__matchers = {}
 
     db.metadata.insert({'level': 'state', '_id': 'ex',
@@ -42,6 +44,15 @@ def test_import_bill():
                          ],
             'documents': [{'title': 'fiscal note',
                           'url': 'http://example.com/fn'}],
+            'actions': [{'action': 'Introduced', 'type': ['bill:introduced'],
+                         'actor': 'upper', 'date': 1331000000},
+            {'action': 'Referred to committee', 'type': ['committee:referred'],
+                         'actor': 'upper', 'date': 1332000000},
+            {'action': 'Passed by voice vote', 'type': ['bill:passed'],
+                         'actor': 'upper', 'date': 1333000000},
+                        {'action': 'Signed', 'type': ['governor:signed'],
+                         'actor': 'governor', 'date': 1334000000},
+                       ],
             'votes': [{'motion': 'passage', 'chamber': 'upper', 'date': None,
                        'yes_count': 1, 'no_count': 1, 'other_count': 0,
                        'yes_votes': ['John Adams'],
@@ -88,13 +99,20 @@ def test_import_bill():
     assert bill['votes'][0]['yes_votes'][0]['leg_id'] == 'EXL000001'
     assert 'committee_id' in bill['votes'][1]
 
+    # test actions
+    assert bill['action_dates']['first'] == 1331000000
+    assert bill['action_dates']['last'] == 1334000000
+    assert bill['action_dates']['passed_upper'] == 1333000000
+    assert bill['action_dates']['signed'] == 1334000000
+    assert bill['action_dates']['passed_lower'] is None
+
     # titles from alternate_titles & versions (not main title)
     assert 'main title' not in bill['alternate_titles']
     assert 'second title' in bill['alternate_titles']
     assert 'old title' in bill['alternate_titles']
 
     # test version/document import
-    assert bill['versions'][0]['doc_id'] == 'EXD00000001'
+    assert_equal(bill['versions'][0]['doc_id'], 'EXD00000001')
     assert bill['versions'][1]['doc_id'] == 'EXD00000002'
     assert bill['documents'][0]['doc_id'] == 'EXD00000003'
 
@@ -136,6 +154,7 @@ def test_import_bill_with_partial_bill_vote_id():
             'versions': [],
             'documents': [],
             'votes': [],
+            'actions': [],
            }
     standalone_votes = {
         # chamber, session, bill id -> vote list
@@ -193,7 +212,7 @@ def test_populate_current_fields():
     assert not b['_current_term']
 
 
-@with_setup(db.vote_ids.drop)
+@with_setup(setup_func)
 def test_votematcher():
     # three votes, two with the same fingerprint
     votes = [{'motion': 'a', 'chamber': 'b', 'date': 'c',
@@ -228,20 +247,23 @@ def test_votematcher():
     assert votes[3]['vote_id'] == 'EXV00000003'
 
 
-@with_setup(db.committees.drop)
+@with_setup(setup_func)
 def test_get_committee_id():
     # 3 committees with the same name, different levels & chamber
     db.committees.insert({'level': 'state', 'state': 'ex', 'chamber': 'upper',
-                          'committee': 'Animal Control', '_id': 'EXC000001'})
+                          'committee': 'Animal Control', 'subcommittee': None,
+                          '_id': 'EXC000001'})
     db.committees.insert({'level': 'state', 'state': 'ex', 'chamber': 'lower',
-                          'committee': 'Animal Control', '_id': 'EXC000002'})
+                          'committee': 'Animal Control', 'subcommittee': None,
+                          '_id': 'EXC000002'})
     db.committees.insert({'level': 'country', 'country': 'zz', 'state': 'ex',
                           'chamber': 'upper',
-                          'committee': 'Animal Control', '_id': 'ZZC000001'})
+                          'committee': 'Animal Control', 'subcommittee': None,
+                          '_id': 'ZZC000001'})
     # committee w/ subcommittee (also has 'Committee on' prefix)
     db.committees.insert({'level': 'state', 'state': 'ex', 'chamber': 'upper',
                           'committee': 'Committee on Science',
-                          '_id': 'EXC000004'})
+                          'subcommittee': None, '_id': 'EXC000004'})
     db.committees.insert({'level': 'state', 'state': 'ex', 'chamber': 'upper',
                           'committee': 'Committee on Science',
                           'subcommittee': 'Space',
