@@ -5,7 +5,11 @@ from billy.importers.metadata import import_metadata, PRESERVED_FIELDS
 from nose.tools import with_setup
 
 
-@with_setup(db.metadata.drop)
+def drop_metadata():
+    db.metadata.drop()
+
+
+@with_setup(drop_metadata)
 def test_import_metadata():
     data_dir = os.path.join(os.path.dirname(os.path.dirname(__file__)),
                             "fixture_data")
