@@ -8,19 +8,15 @@ from collections import defaultdict
 from django.http import HttpResponse
 
 from billy import db
+from billy.models import Bill
 from billy.conf import settings
-from billy.utils import metadata, find_bill
+from billy.utils import metadata, find_bill, parse_param_dt
 
 import pymongo
-import pyes
 
 from piston.utils import rc
 from piston.handler import BaseHandler, HandlerMetaClass
 
-from jellyfish import levenshtein_distance
-
-elasticsearch = pyes.ES(settings.ELASTICSEARCH_HOST,
-                        settings.ELASTICSEARCH_TIMEOUT)
 
 _chamber_aliases = {
     'assembly': 'lower',
@@ -29,16 +25,6 @@ _chamber_aliases = {
 }
 
 
-def parse_param_dt(dt):
-    formats = ['%Y-%m-%d %H:%M',    # here for legacy reasons
-               '%Y-%m-%dT%H:%M:%S',
-               '%Y-%m-%d']
-    for format in formats:
-        try:
-            return datetime.datetime.strptime(dt, format)
-        except ValueError:
-            pass
-
 _lower_fields = ('state',)
 
 
@@ -171,84 +157,28 @@ class BillSearchHandler(BillyHandler):
         bill_fields = _build_field_list(request, bill_fields)
 
         # normal mongo search logic
-        _filter = _build_mongo_filter(request, ('state', 'chamber',
-                                                'subjects', 'bill_id',
-                                                'bill_id__in'))
+        base_fields = _build_mongo_filter(request, ('state', 'chamber',
+                                                    'subjects', 'bill_id',
+                                                    'bill_id__in'))
 
-        # process search_window
-        search_window = request.GET.get('search_window', '')
-        if search_window:
-            if search_window == 'session':
-                _filter['_current_session'] = True
-            elif search_window == 'term':
-                _filter['_current_term'] = True
-            elif search_window.startswith('session:'):
-                _filter['session'] = search_window.split('session:')[1]
-            elif search_window.startswith('term:'):
-                _filter['_term'] = search_window.split('term:')[1]
-            elif search_window == 'all':
-                pass
-            else:
-                resp = rc.BAD_REQUEST
-                resp.write(": invalid search_window. Valid choices are "
-                           "'term', 'session' or 'all'")
-                return resp
-
-        # process updated_since
-        since = request.GET.get('updated_since')
-        if since:
-            try:
-                _filter['updated_at'] = {'$gte': parse_param_dt(since)}
-            except ValueError:
-                resp = rc.BAD_REQUEST
-                resp.write(": invalid updated_since parameter."
-                           " Please supply a date in YYYY-MM-DD format.")
-                return resp
-
-        # process sponsor_id
-        sponsor_id = request.GET.get('sponsor_id')
-        if sponsor_id:
-            _filter['sponsors.leg_id'] = sponsor_id
-
-        # process full-text query
+        # process extra attributes
         query = request.GET.get('q')
-        if query:
-            query = {"query_string": {"fields": ["text", "title"],
-                                                "query": query}}
-            search = pyes.Search(query, fields=[])
+        search_window = request.GET.get('search_window', 'all')
+        since = request.GET.get('updated_since', None)
+        sponsor_id = request.GET.get('sponsor_id')
 
-            # take terms from mongo query
-            es_terms = []
-            if 'state' in _filter:
-                es_terms.append(pyes.TermFilter('state',
-                                                _filter.pop('state')))
-            if 'session' in _filter:
-                es_terms.append(pyes.TermFilter('session',
-                                                _filter.pop('session')))
-            if 'chamber' in _filter:
-                es_terms.append(pyes.TermFilter('chamber',
-                                                _filter.pop('chamber')))
-            if 'subjects' in _filter:
-                es_terms.append(pyes.TermFilter('subjects',
-                                           _filter.pop('subjects')['$all']))
-            if 'sponsors.leg_id' in _filter:
-                es_terms.append(pyes.TermFilter('sponsors',
-                                                _filter.pop('sponsors.leg_id')))
+        try:
+            query = Bill.search(query,
+                                search_window=search_window,
+                                updated_since=since, sponsor_id=sponsor_id,
+                                bill_fields=bill_fields,
+                                **base_fields)
+        except ValueError as e:
+            resp = rc.BAD_REQUEST
+            resp.write(': %s' % e)
+            return resp
 
-            # add terms
-            if es_terms:
-                search.filter = pyes.ANDFilter(es_terms)
-
-            # page size is a guess, could use tweaks
-            es_result = elasticsearch.search(search, search_type='scan',
-                                             scroll='3m', size=250)
-            doc_ids = [r.get_id() for r in es_result]
-            _filter['versions.doc_id'] = {'$in': doc_ids}
-
-        # start with base query
-        query = db.bills.find(_filter, bill_fields)
-
-        # pagination
+        # add pagination
         page = request.GET.get('page')
         per_page = request.GET.get('per_page')
         if page and not per_page:
@@ -259,11 +189,10 @@ class BillSearchHandler(BillyHandler):
         if page:
             page = int(page)
             per_page = int(per_page)
-            query = query.limit(per_page).skip(per_page*(page-1))
+            query = query.limit(per_page).skip(per_page * (page - 1))
         else:
             # limit response size
-            count = db.bills.find(_filter, bill_fields).count()
-            if count > 5000:
+            if query.count() > 10000:
                 resp = rc.BAD_REQUEST
                 resp.write(': request too large, try narrowing your search by '
                            'adding more filters.')
@@ -275,6 +204,8 @@ class BillSearchHandler(BillyHandler):
             query = query.sort([('updated_at', -1)])
         elif sort == 'created_at':
             query = query.sort([('created_at', -1)])
+        elif sort == 'last_action':
+            query = query.sort([('action_dates.last', -1)])
 
         return list(query)
 
@@ -394,114 +325,6 @@ class SubjectListHandler(BillyHandler):
         return result
 
 
-class ReconciliationHandler(BaseHandler):
-    """
-    An endpoint compatible with the Google Refine 2.0 reconciliation API.
-
-    Given a query containing a legislator name along with some optional
-    filtering properties ("state", "chamber"), this handler will return
-    a list of possible matching legislators.
-
-    See http://code.google.com/p/google-refine/wiki/ReconciliationServiceApi
-    for the API specification.
-    """
-    allowed_methods = ('GET', 'POST')
-    key = getattr(settings, 'SUNLIGHT_SERVICES_KEY', 'no-key')
-
-    metadata = {
-        "name": "Billy Reconciliation Service",
-        "view": {
-            "url": "%slegislators/preview/{{id}}/?apikey=%s" % (
-                settings.API_BASE_URL, key),
-            },
-        "preview": {
-            "url": "%slegislators/preview/{{id}}/?apikey=%s" % (
-                settings.API_BASE_URL, key),
-            "width": 430,
-            "height": 300
-            },
-        "defaultTypes": [
-            {"id": "/billy/legislator",
-             "name": "Legislator"}],
-        }
-
-    def read(self, request):
-        return self.reconcile(request)
-
-    def create(self, request):
-        return self.reconcile(request)
-
-    def reconcile(self, request):
-        query = request.GET.get('query') or request.POST.get('query')
-
-        if query:
-            # If the query doesn't start with a '{' then it's a simple
-            # string that should be used as the query w/ no extra params
-            if not query.startswith("{"):
-                query = '{"query": "%s"}' % query
-            query = json.loads(query)
-
-            return {"result": self.results(query)}
-
-        # Batch query mode
-        queries = request.GET.get('queries') or request.POST.get('queries')
-
-        if queries:
-            queries = json.loads(queries)
-            ret = {}
-            for key, value in queries.items():
-                ret[key] = {'result': self.results(value)}
-
-            return ret
-
-        # If no queries, return metadata
-        return self.metadata
-
-    def results(self, query):
-        # Look for the query to be a substring of a legislator name
-        # (case-insensitive)
-        pattern = re.compile(".*%s.*" % query['query'],
-                             re.IGNORECASE)
-
-        spec = {'full_name': pattern}
-
-        for prop in query.get('properties', []):
-            # Allow filtering by state or chamber for now
-            if prop['pid'] in ('state', 'chamber'):
-                spec[prop['pid']] = prop['v']
-
-        legislators = db.legislators.find(spec)
-
-        results = []
-        for leg in legislators:
-            if legislators.count() == 1:
-                match = True
-                score = 100
-            else:
-                match = False
-                if leg['last_name'] == query['query']:
-                    score = 90
-                else:
-                    distance = levenshtein_distance(leg['full_name'].lower(),
-                                                    query['query'].lower())
-                    score = 100.0 / (1 + distance)
-
-            # Note: There's a bug in Refine that causes reconciliation
-            # scores to be overwritten if the same legislator is returned
-            # for multiple queries. see:
-            # http://code.google.com/p/google-refine/issues/detail?id=185
-
-            results.append({"id": leg['_id'],
-                            "name": leg['full_name'],
-                            "score": score,
-                            "match": match,
-                            "type": [
-                                {"id": "/billy/legislator",
-                                 "name": "Legislator"}]})
-
-        return sorted(results, cmp=lambda l, r: cmp(r['score'], l['score']))
-
-
 class LegislatorGeoHandler(BillyHandler):
     base_url = settings.BOUNDARY_SERVICE_URL
 
@@ -513,8 +336,8 @@ class LegislatorGeoHandler(BillyHandler):
             resp.write(': Need lat and long parameters')
             return resp
 
-        url = "%sboundary/?shape_type=none&contains=%s,%s&sets=sldl,sldu&limit=0" % (
-            self.base_url, latitude, longitude)
+        url = ("%sboundary/?shape_type=none&contains=%s,%s&sets=sldl,sldu"
+               "&limit=0" % (self.base_url, latitude, longitude))
 
         resp = json.load(urllib2.urlopen(url))
 
@@ -535,7 +358,8 @@ class LegislatorGeoHandler(BillyHandler):
                 filters.append({'state': state,
                                 'district': districts[0]['name'],
                                 'chamber': chamber})
-                boundary_mapping[(state, districts[0]['name'], chamber)] = census_name
+                boundary_mapping[(state, districts[0]['name'],
+                                  chamber)] = census_name
 
         if not filters:
             return []
